<?php
/* --------------------------------------------------------------
   AfterbuyCronjobTask.inc.php 2022-04-19
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/
declare(strict_types=1);

use GXModules\Gambio\Afterbuy\Admin\Classes\AfterbuyXML\AfterbuyXMLService;
use GXModules\Gambio\Afterbuy\Admin\Classes\AfterbuyXML\Exceptions\XMLException;
use GXModules\Gambio\Afterbuy\Admin\Classes\LoggerAdapter;
use GXModules\Gambio\Afterbuy\Admin\Classes\ProductsQuantityUpdateService;

class AfterbuyCronjobTask extends AbstractCronjobTask
{
    protected const MAX_ORDERS_PER_RUN = 20;
    
    /**
     * @param float $cronjobStartAsMicrotime
     *
     * @return Closure
     */
    public function getCallback($cronjobStartAsMicrotime)
    {
        return function () {
            $this->logger->lastRun();
            
            if (!$this->moduleIsInstalledAndActive()) {
                return true;
            }
            
            if ($this->dependencies->getDependencies()['sendOrdersActive']) {
                $this->sendOrders();
            }
            if ($this->dependencies->getDependencies()['updateQuantitiesActive']) {
                $this->updateProductsQuantities();
            }
            
            $this->logger->log(['CronjobTask finished' => date('c')]);
            $this->logger->lastSuccess();
            
            return true;
        };
    }
    
    
    protected function updateProductsQuantities(): void
    {
        /** @var GambioAfterbuyConfigurationStorage $configurationStorage */
        $configurationStorage = $this->dependencies->getDependencies()['ConfigurationStorage'];
        $lastSyncDateTime = new \DateTimeImmutable($configurationStorage->get('last_qty_sync'));
        $newLastSyncDateTime = new \DateTimeImmutable();
        $this->logInfo("Syncing quantity updates since {$lastSyncDateTime->format('c')}");        
        /** @var AfterbuyXMLService $xmlService */
        $xmlService = $this->dependencies->getDependencies()['XMLService'];
        try {
            $updatedProducts = $xmlService->getShopProductsModifiedSince($lastSyncDateTime);
            if (empty($updatedProducts)) {
                $this->logInfo("No updated products found at Afterbuy.");
            } else {
                $this->logInfo("Processing " . count($updatedProducts) . " products received from Afterbuy");
                /** @var ProductsQuantityUpdateService $productsQuantitiesUpdateService */
                $productsQuantitiesUpdateService = $this->dependencies->getDependencies()['ProductsQuantitiesUpdateService'];
                $productsQuantitiesUpdateService->setLogger(new LoggerAdapter($this->logger));
                $productSyncType = $configurationStorage->get('product_sync_type');
                $productsQuantitiesUpdateService->updateProductQuantities($productSyncType, ...$updatedProducts);
            }
            $configurationStorage->set('last_qty_sync', $newLastSyncDateTime->format('c'));
        } catch (XMLException $e) {
            $this->logError("ERROR: {$e->getMessage()}");
        }
    }
    
    
    public function sendOrders(): void
    {
        /** @var GambioAfterbuyConfigurationStorage $configurationStorage */
        $configurationStorage = $this->dependencies->getDependencies()['ConfigurationStorage'];
        $allAsPaid    = $configurationStorage->get('order_status_paid') === '-1';
        $responseData = [];
        $headers      = [];
        $maxOrderPerRun = (int)$configurationStorage->get('max_orders_per_run');
        $maxOrderPerRun = $maxOrderPerRun > 0 ?: static::MAX_ORDERS_PER_RUN;
        
        $this->logInfo('Cron endpoint sending up to ' . $maxOrderPerRun . ' orders');
        $db                = StaticGXCoreLoader::getDatabaseQueryBuilder();
        $queuedOrdersQuery = $db->select('orders_id')
            ->where('afterbuy_success', '0')
            ->order_by('orders_id DESC')
            ->limit($maxOrderPerRun)
            ->get('orders');
        $queuedOrdersIds   = [];
        foreach ($queuedOrdersQuery->result() as $queuedOrder) {
            $queuedOrdersIds[] = $queuedOrder->orders_id;
        }
        sort($queuedOrdersIds);
        $responseData['processedOrders'] = [];
        $responseData['unpaidOrders']    = [];
        $responseData['errors']          = [];
        foreach ($queuedOrdersIds as $orderId) {
            $afterbuyOrderSender = MainFactory::create('AfterbuyOrderSender', $orderId);
            if ($allAsPaid || $afterbuyOrderSender->orderIsPaid()) {
                try {
                    $afterbuyOrderSender->processOrder();
                    $responseData['processedOrders'][] = $orderId;
                } catch (AfterbuyException $e) {
                    $responseData['errors'][$orderId] = $e->getMessage();
                }
            } else {
                $responseData['unpaidOrders'][] = $orderId;
            }
        }
        
        $resultLog = sprintf("Cron job summary:\nProcessed orders: %s\nUnpaid orders (skipped): %s",
                             implode(', ', $responseData['processedOrders']),
                             implode(', ', $responseData['unpaidOrders']));
        if (!empty($responseData['errors'])) {
            $resultLog .= sprintf("\nOrders with errors in transmission:\n");
            foreach ($responseData['errors'] as $errorOrder => $errorMessage) {
                $resultLog .= sprintf("%d - %s\n", $errorOrder, $errorMessage);
            }
        }
        $this->logInfo($resultLog);
    }
    
    
    
    protected function logInfo(string $message): void
    {
        $this->logger->log(['message' => $message, 'level' => 'info']);
    }
    
    protected function logError(string $message): void
    {
        $this->logger->logError(['message' => $message, 'level' => 'error']);
    }

    protected function moduleIsInstalledAndActive(): bool
    {
        $configurationFinder = $this->dependencies->getDependencies()['ConfigurationFinder'];
        $installedConfig     = (bool)$configurationFinder->get('gm_configuration/MODULE_CENTER_GAMBIOAFTERBUY_INSTALLED');
        $activeConfig        = (bool)$configurationFinder->get('modules/gambio/afterbuy/active');
        
        return $installedConfig && $activeConfig;
    }
}