<?php
/* --------------------------------------------------------------
   AfterbuyXMLService.inc.php 2022-04-11
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/
declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\Admin\Classes\AfterbuyXML;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\TransferException;
use GXModules\Gambio\Afterbuy\Admin\Classes\AfterbuyXML\Exceptions\XMLException;
use function Gambio\Core\Logging\logger;

class AfterbuyXMLService
{
    private $partnerToken;
    
    private $accountToken;
    
    private const BASE_URI          = 'https://api.afterbuy.de/afterbuy/ABInterface.aspx';
    private const LOGGING_NAMESPACE = 'AfterbuyXML';
    
    
    public function __construct(string $partnerToken, string $accountToken)
    {
        $this->partnerToken = $partnerToken;
        $this->accountToken = $accountToken;
    }
    
    
    /**
     * Performs a call to Afterbuy’s XML API.
     *
     * @param string $callName
     * @param array  $callData
     *
     * @return \SimpleXMLElement
     * @throws XMLException
     */
    public function performCall(string $callName, array $callData = []): \SimpleXMLElement
    {
        logger(static::LOGGING_NAMESPACE)->debug("Calling {$callName}");
        $client                 = new Client([
                                                 'base_uri' => static::BASE_URI,
                                                 'timeout'  => 5.0,
                                             ]);
        $xmlBodyData            = [
            'Request' => [
                'AfterbuyGlobal' => [
                    'PartnerToken' => $this->partnerToken,
                    'AccountToken' => $this->accountToken,
                    'CallName'     => $callName,
                ],
            ],
        ];
        $xmlBodyData['Request'] = array_merge($callData, $xmlBodyData['Request']);
        $xmlBody                = static::arrayToXML($xmlBodyData);
        //die($xmlBody);
        try {
            $response     = $client->post(static::BASE_URI, [
                'body'    => $xmlBody,
                'headers' => [
                    'Content-Type' => 'application/xml',
                ],
            ]);
            $responseBody = $response->getBody()->getContents();
            
            return static::parseXMLResponse($responseBody);
        } catch (TransferException $transferException) {
            logger(static::LOGGING_NAMESPACE)->error("ERROR calling {$callName}: {$transferException->getMessage()} ({$transferException->getCode()})");
            throw new XMLException($transferException->getMessage());
        }
    }
    
    
    /**
     * @param string $responseBody
     *
     * @return \SimpleXMLElement
     * @throws XMLException
     */
    private static function parseXMLResponse(string $responseBody): \SimpleXMLElement
    {
        $xmlErrorMode = libxml_use_internal_errors(true);
        libxml_clear_errors();
        $xmlResponse = simplexml_load_string($responseBody);
        $xmlErrors   = libxml_get_errors();
        libxml_use_internal_errors($xmlErrorMode);
        if (!empty($xmlErrors)) {
            foreach ($xmlErrors as $xmlError) {
                if ($xmlError->level === LIBXML_ERR_WARNING) {
                    logger(static::LOGGING_NAMESPACE)->warning("XML decoding: {$xmlError->message} in line {$xmlError->line}");
                } else {
                    $errorMessage = "API response cannot be processed: {$xmlError->message} (line {$xmlError->line})";
                    logger(static::LOGGING_NAMESPACE)->error($errorMessage);
                    throw new XMLException($errorMessage);
                }
            }
        }
        
        return $xmlResponse;
    }
    
    
    /**
     * Retrieves server time from Afterbuy and returns it as a DateTimeImmutable instance.
     *
     * @return \DateTimeImmutable
     * @throws XMLException
     */
    public function getAfterbuyDatetime(): \DateTimeImmutable
    {
        $xml       = $this->performCall('GetAfterbuyTime');
        $abUtcTime = (string)$xml->Result->AfterbuyUniversalTimeStamp;
        try {
            $dateTime = new \DateTimeImmutable($abUtcTime, new \DateTimeZone('UTC'));
        } catch (\Exception $e) {
            throw new XMLException($e->getMessage());
        }
        
        return $dateTime;
    }
    
    
    /**
     * @param \DateTimeInterface $startDate
     *
     * @return array
     * @throws XMLException
     */
    public function getShopProductsModifiedSince(\DateTimeInterface $startDate): array
    {
        $lastProductId   = null;
        $hasMoreProducts = true;
        $products        = [];
        while ($hasMoreProducts) {
            $callParams = [
                'MaxShopItems' => 100,
                'DataFilter'   => [
                    'Filter' => [
                        [
                            'FilterName'   => 'DateFilter',
                            'FilterValues' => [
                                'DateFrom'    => $startDate->format('Y-m-d H:i:s'),
                                'FilterValue' => 'ModDate',
                            ],
                        ],
                    ],
                ],
            ];
            if ($lastProductId !== null) {
                $callParams['DataFilter']['Filter'][] = [
                    'FilterName'   => 'RangeID',
                    'FilterValues' => [
                        'ValueFrom' => $lastProductId + 1,
                    ],
                ];
            }
            $xml = $this->performCall('GetShopProducts', $callParams);
            if ((string)$xml->CallStatus === 'Error' && (string)$xml->Result->ErrorList->Error->ErrorCode === '15') {
                // not really an error, just an empty result
                return [];
            }
            if ((string)$xml->CallStatus !== 'Success' || count($xml->Result->Products->Product) === 0) {
                throw new XMLException("getShopProductsModifiedSince {$startDate->format('c')} failed");
            }
            
            /** @var \SimpleXMLElement $product */
            foreach ($xml->Result->Products->Product as $product) {
                $abProduct = new AfterbuyProduct((string)$product->EAN, (string)$product->Name);
                $abProduct->setQuantity((int)$product->Quantity);
                $abProduct->setAnr((float)$product->Anr);
                $abProduct->setProductID((int)$product->ProductID);
                $products[] = $abProduct;
            }
            
            $hasMoreProducts = (bool)(string)$xml->Result->HasMoreProducts;
            if ($hasMoreProducts) {
                $lastProductId = (int)$xml->Result->LastProductID;
            }
        }
        
        return $products;
    }
    
    
    /**
     * @param AfterbuyProduct ...$products
     *
     * @return \SimpleXMLElement
     * @throws XMLException
     */
    public function updateProducts(AfterbuyProduct ...$products): \SimpleXMLElement
    {
        $productsArray = [];
        foreach ($products as $product) {
            $productArray                 = $product->toArray();
            $productArray['ProductIdent'] = [
                'ProductInsert' => 1,
                'EAN'           => $product->getEAN(),
            ];
            $productsArray[]              = $productArray;
        }
        $callParams = [
            'Products' => [
                'Product' => $productsArray,
            ],
        ];
        $xml        = $this->performCall('UpdateShopProducts', $callParams);
        
        return $xml;
    }
    
    
    /**
     * @param array  $input
     * @param int    $indent
     * @param string $indentChar
     *
     * @return string
     */
    public static function arrayToXML(array $input, int $indent = 0, string $indentChar = '    '): string
    {
        $output      = '';
        $indentation = str_repeat($indentChar, $indent);
        foreach ($input as $key => $value) {
            if (is_string($value)) {
                $output .= "{$indentation}<{$key}><![CDATA[{$value}]]></{$key}>\n";
            }
            if (is_numeric($value)) {
                $output .= "{$indentation}<{$key}>{$value}</{$key}>\n";
            }
            if (is_array($value)) {
                if (static::arrayIsList($value)) {
                    foreach ($value as $element) {
                        if (is_array($element)) {
                            $output .= "{$indentation}<{$key}>\n" . static::arrayToXML($element, $indent + 1)
                                       . "{$indentation}</{$key}>\n";
                        }
                        if (is_string($element)) {
                            $output .= "{$indentation}<{$key}>{$element}</{$key}>\n";
                        }
                    }
                } else {
                    $output .= "{$indentation}<{$key}>\n" . static::arrayToXML($value, $indent + 1)
                               . "{$indentation}</{$key}>\n";
                }
            }
        }
        
        return $output;
    }
    
    
    public static function arrayIsList(array $input): bool
    {
        if (function_exists('array_is_list')) {
            return array_is_list($input);
        }
        
        return array_keys($input)[0] === 0; // enough for this purpose, but not a general solution!
    }
}