<?php
/* --------------------------------------------------------------
   ProductsQuantityUpdateService.php 2022-04-21
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/
declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\Admin\Classes;

use GXModules\Gambio\Afterbuy\Admin\Classes\AfterbuyXML\AfterbuyProduct;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;

require_once(DIR_FS_CATALOG.'gm/inc/set_shipping_status.php');

/**
 * Updates product quantities.
 * 
 * This class does not use ProductReadService and ProductWriteService due to performance issues. 
 */
class ProductsQuantityUpdateService implements LoggerAwareInterface
{
    /** @var \CI_DB_query_builder */
    protected $db;
    
    /**
     * @var LoggerInterface
     */
    private $logger;
    
    private $result_array;
    
    
    public function __construct(\CI_DB_query_builder $db)
    {
        $this->db = $db;
    }
    
    
    /**
     * Uses a set of AfterbuyProducts to update local product quantities.
     *
     * Uses the EAN (External Article Number) from Afterbuy to identify local products by their products_id.
     * 
     * @param AfterbuyProduct ...$products
     *
     * @return void
     */
    public function updateProductQuantities(string $mode = 'pid_ean', AfterbuyProduct ...$products): void
    {
        $this->log("Match mode: {$mode}");
        foreach ($products as $product) {
            if ($mode === 'pid_ean') {
                $exan       = $product->getEAN();
                $productsId = (int)$exan;
                if ($exan !== (string)$productsId) {
                    $this->log("Product with EXAN \"{$exan}\" cannot be syncd, skipping");
                    continue;
                }
            } elseif ($mode === 'pid_anr') {
                $productsId = (int)$product->getAnr();
                if (empty($productsId)) {
                    $this->log("Product with Afterbuy ProductsID {$product->getProductID()} has no usable Anr ({$product->getAnr()}), skipping");
                    continue;
                }
            } else {
                if ($mode === 'model_ean') {
                    $model = $product->getEAN();
                } elseif ($mode === 'model_anr') {
                    $model = (string)$product->getAnr();
                } elseif ($mode === 'model_prodid') {
                    $model = (string)$product->getProductID();
                }
                if (empty($model)) { // N. B.: This intentionally covers $model === '0'!
                    $this->log("Product with Afterbuy ProductsID {$product->getProductID()} has no usable identification, skipping");
                    continue;
                }
                $productsIds = $this->findProductsIdsByModel($model);
                if (empty($productsIds)) {
                    $this->log("Product model {$model} not found, skipping");
                    continue;
                }
                if (count($productsIds) > 1) {
                    $this->log("Product model \"{$model}\" is ambiguous, skipping");
                    continue;
                }
                $productsId = (int)$productsIds[0];
            }
            if (!$this->isValidProductsId($productsId)) {
                $this->log("Product with products_id {$productsId} does not exist, skipping.");
                continue;
            }
            if ($this->productHasAttributes($productsId) || $this->productHasProperties($productsId)) {
                $this->log("Product with products_id {$productsId} has attributes/properties, skipping.");
                continue;
            }
            $this->log("Setting quantity of product {$productsId} ({$product->getName()}) to {$product->getQuantity()}.");
            $this->setProductQuantity($productsId, (float)$product->getQuantity());
        }
    }
    
    protected function setProductQuantity(int $productsId, float $quantity)
    {
        $this->db->update('products', ['products_quantity' => $quantity], ['products_id' => $productsId]);
        set_shipping_status($productsId, null, $quantity);
    }
    
    protected function isValidProductsId(int $productsId): bool
    {
        $rows = $this->db->get_where('products', ['products_id' => $productsId])->num_rows();
        $this->db->reset_query();
        return $rows === 1;
    }
    
    protected function productHasAttributes(int $productsId): bool
    {
        $numAttributesRow = $this->db->select('COUNT(*) AS num_attribs')
            ->from('products_attributes')
            ->where('products_id', $productsId)
            ->get()
            ->row_array();
        $numAttributes = $numAttributesRow['num_attribs'];
        return $numAttributes > 0;
    }
    
    protected function productHasProperties(int $productsId): bool
    {
        $numPropertiesRow = $this->db->select('COUNT(*) AS num_combis')
            ->from('products_properties_combis')
            ->where('products_id', $productsId)
            ->get()
            ->row_array();
        $numProperties = $numPropertiesRow['num_combis'];
        return $numProperties > 0;
    }
    
    protected function findProductsIdsByModel(string $model): array
    {
        $rows = $this->result_array = $this->db->select('products_id')
            ->get_where('products', ['products_model' => $model])
            ->result_array();
        $productIds = [];
        foreach ($rows as $row) {
            $productIds[] = $row['products_id'];
        }
        return $productIds;
    }
    
    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }
    
    protected function log(string $message): void
    {
        if ($this->logger !== null) {
            $this->logger->info($message);
        }
    }
}